/*
  Temporizador.h - Librería para Microcontrolador tipo Industrial
  Copyright (c) 2018 FaradayBits. Todos los derechos reservados.

  Autor: FaradayBits
  Versiones: 0.0.1

  v0.0.1 - May 15, 2018 - Versión inicial

  FM24I2C
    https://github.com/nw-wind/FM24I2C

  RTClib
    https://github.com/adafruit/RTClib
    Direcciones de memoria 0x07-0x0D (0-6)
*/

#ifndef Temporizador_H
#define Temporizador_H

#include <Arduino.h>
#include <EEPROM.h>
#include <Wire.h>
#include "RTClib.h"

#define STATUS_RTC_ERROR        1
#define STATUS_DESHABILITADO    2
#define STATUS_INICIANDO        3
#define STATUS_INICIO_ESPERA    4
#define STATUS_ARRANQUE         5
#define STATUS_ARRANQUE_ESPERA  6
#define STATUS_ENCENDIDO        7
#define STATUS_ENCENDIDO_ESPERA 8

#define RTC_DEFAULT_ADDRESS   0x68
#define FMRAM_DEFAULT_ADDRESS 0x57

#define NVRAM_MODE 1
#define FMRAM_MODE 2

class Temporizador
{
  private:
    // Numero de instancias
    static uint8_t _instancias_num;
    uint8_t _instancia_num;

    // Direcciones de Hardware
    static uint8_t _ds3231_i2c_address;
    static uint8_t _fm24c64_i2c_address;
    uint8_t _relay_pin;

    // EEPROM ADDRESS
    uint8_t _eeprom_address_inicio_automatico;
    uint8_t _eeprom_address_reanudar_encendido;

    uint8_t _eeprom_address_tiempo_inicio_espera;
    uint8_t _eeprom_address_tiempo_arranque;
    uint8_t _eeprom_address_tiempo_arranque_espera;
    uint8_t _eeprom_address_tiempo_activo;
    uint8_t _eeprom_address_tiempo_activo_espera;

    // DS3231/FM24C64 ADDRESS
    uint8_t _memory_address_periodo;
    uint8_t _memory_address_periodo_time;
    uint8_t _memory_mode;

    // Uso del temporizador
    uint8_t _periodo;
    int32_t _periodo_time;

    boolean  _detenido;
    boolean  _status_relay;

    // Contador para ciclo
    uint32_t _T[3];


    //** MÉTODOS PRIVADOS**
    void _init();
    void _setup(uint8_t me_mo, uint8_t ti_in_es, uint8_t ti_ar, uint8_t ti_ar_es, uint8_t ti_ac, uint8_t ti_ac_es, uint8_t st_te, uint8_t pe_ti, uint8_t re_pi);

    void     EEPROMWritelong(uint8_t address, uint32_t value);
    uint32_t EEPROMReadlong(uint8_t address);

    void     writeMemory(uint8_t addr, uint8_t values);
    void     writeMemory(uint8_t addr, uint32_t values, uint8_t nBytes);
    uint8_t  readMemory(uint8_t addr);
    uint32_t readMemory(uint8_t addr, uint16_t nBytes);

    void proximoPeriodo();


  public:
    // Fecha y hora, Reloj de Tiempo Real
    static DateTime _now;
    static RTC_DS3231 _RTC;

    //** CONSTRUCTOR **
    Temporizador(uint8_t me_mo, uint8_t ti_in_es, uint8_t ti_ar, uint8_t ti_ar_es, uint8_t ti_ac, uint8_t ti_ac_es, uint8_t st_te, uint8_t pe_ti, uint8_t re_pi);
    Temporizador(uint8_t me_mo, uint8_t i_aut, uint8_t r_enc, uint8_t ti_in_es, uint8_t ti_ar, uint8_t ti_ar_es, uint8_t ti_ac, uint8_t ti_ac_es, uint8_t st_te, uint8_t pe_ti, uint8_t re_pi);


    //** DESTRUCTOR **
    ~Temporizador();

    // Set EEPROM address
    void setEepromAddressInicioAutomatico(uint8_t address);
    void setEepromAdressReanudarEncendido(uint8_t address);
    void setEepromAddressTiempoInicioEspera(uint8_t address);
    void setEepromAddressTiempoArranque(uint8_t address);
    void setEepromAddressTiempoArranqueEspera(uint8_t address);
    void setEepromAddressTiempoEncendido(uint8_t address);
    void setEepromAddressTiempoEncendidoEspera(uint8_t address);

    // Set RTC Address
    void setMemoryAddressPeriodo(uint8_t address);
    void setMemoryAddressPeriodoTime(uint8_t address);

    // Set Hardware Address
    void setRelayPin(uint8_t relay_pin);
    void setMemoryMode(uint8_t memory_mode);
    void setRtcAdress(uint8_t address);
    void setFmramAdress(uint8_t address);

    // Configuraciones Temporizador
    void setInicioAutomatico(uint8_t inicio_automatico);
    void setReanudarEncendido(uint8_t reanudar_encendido);

    void setTiempoInicioEspera(uint32_t segundos);
    void setTiempoInicioEspera(uint8_t minutos, uint8_t segundos);
    void setTiempoArranque(uint32_t segundos);
    void setTiempoArranque(uint8_t horas, uint8_t minutos, uint8_t segundos);
    void setTiempoArranqueEspera(uint32_t segundos);
    void setTiempoArranqueEspera(uint8_t horas, uint8_t minutos, uint8_t segundos);
    void setTiempoEncendido(uint32_t segundos);
    void setTiempoEncendido(uint8_t horas, uint8_t minutos, uint8_t segundos);
    void setTiempoEncendidoEspera(uint32_t segundos);
    void setTiempoEncendidoEspera(uint8_t horas, uint8_t minutos, uint8_t segundos);

    // Set Fecha y Hora
    void setFecha(uint16_t anho, uint8_t mes, uint8_t dia);
    void setHora(uint8_t hrs, uint8_t min, uint8_t seg);
    void setFechaHora(uint16_t anho, uint8_t mes, uint8_t dia, uint8_t hrs, uint8_t min, uint8_t seg);

    // Get Fecha y Hora
    String getFecha();
    String getHora();

    // Status Temporizador
    uint8_t  getInstanciasNum();
    uint8_t  getInstanciaNum();
    uint8_t  getPeriodo();
    uint32_t getPeriodoTiempo();
    boolean  getStatusRelay();
    boolean  getStatusDetenido();

    boolean  getInicioAutomatico();
    boolean  getReanudarInicio();

    uint32_t getTiempoInicioEspera();
    uint32_t getTiempoArranque();
    uint32_t getTiempoArranqueEspera();
    uint32_t getTiempoEncendido();
    uint32_t getTiempoEncendidoEspera();

    // Metodos de control
    void setDefaults();
    void reiniciar();
    void deshabilitar();
    void detener();
    void reanudar();

    // PROCESO DEL TEMPORIZADOR
    void inizializar();
    void procesarTemporizador();

};

#endif